﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;

namespace CustomColumnNamespace
{
    public class DiacriticInsensitiveColumn : GridBoundColumn
    {
        //RadGrid will call this method when the value should be set to the filtering input control(s)
        protected override void SetCurrentFilterValueToControl(TableCell cell)
        {
            if (this.CurrentFilterFunction != GridKnownFunction.NoFilter)
            {
                TextBox textBox = (TextBox)cell.Controls[0];
                if (string.IsNullOrWhiteSpace(textBox.Text))
                {
                    textBox.Text = ViewState["FilterValuePreserver" + this.UniqueName] as string;
                }
            }
        }

        // Building the filter expression for the custom column, which will be added to the final FilterExpression of the GridTableView
        public override string EvaluateFilterExpression(GridFilteringItem filteringItem)
        {
            if (!this.SupportsFiltering())
            {
                return string.Empty;
            }

            TableCell cell = filteringItem[this.UniqueName];
            string filterValue = this.GetCurrentFilterValueFromControl(cell);
            ViewState["FilterValuePreserver" + this.UniqueName] = filterValue;

            if (String.IsNullOrEmpty(filterValue) ||
                this.CurrentFilterFunction == GridKnownFunction.NoFilter)
            {
                return string.Empty;
            }
            else
            {
                TextBox textBox = (TextBox)cell.Controls[0];
                string[] diacritics = new string[] { 
                    "aâãäàá",
                    "eêëèé",
                    "iîïìí",
                    "oôõöòó",
                    "uûüùú",
                    "yý",
                    "nñ",
                    "cç"
                };

                string v = filterValue.ToLowerInvariant().Trim();
                HashSet<string> values = new HashSet<string>() { v };
                HashSet<string> valuesTemp = new HashSet<string>();

                for (int i = 0; i < v.Length; i++)
                {
                    char letter = v[i];
                    foreach (string letterGroup in diacritics)
                    {
                        if (letterGroup.Contains(letter))
                        {
                            foreach (char diacritic in letterGroup)
                            {
                                foreach (string value in values)
                                {
                                    char[] result = value.ToCharArray();
                                    result[i] = diacritic;
                                    valuesTemp.Add(new string(result));
                                }
                            }
                            values.UnionWith(valuesTemp);
                        }
                    }
                }
                List<string> expressions = new List<string>();

                foreach (string value in values)
                {
                    textBox.Text = value;
                    expressions.Add("(" + base.EvaluateFilterExpression(filteringItem) + ")");
                }
                return string.Join("OR", expressions);
            }
        }
    }
}